Attribute VB_Name = "URUtils"
' URSchedule v1.0.0

Sub URSchedule()
    frmURSchedule.Show

End Sub

Sub AddURAppts(varStart As Variant, iDefaultImportance As OlImportance, iBusyStatus As OlBusyStatus, iDefaultDuration As Integer, _
    abValidDays() As Boolean, varBeginDay As Variant, varEndDay As Variant)
        
    Dim varEnd As Variant
    
    ' Get text from clipboard and identify separator character
    Dim myOlApp As Application
    Dim myNameSpace As NameSpace
    Dim myData As DataObject
    
    Set myOlApp = CreateObject("Outlook.Application")
    Set myNameSpace = myOlApp.GetNamespace("MAPI")
    Set myData = New DataObject
    
    Dim strClip, strSep, strTask, strTitle, strText As String
    Dim iPriority As Integer
    Dim iImportance As OlImportance
    Dim iSepLength As Integer
    
    myData.GetFromClipboard
    strClip = myData.GetText(1)
    strSep = Chr(13) + Chr(10) ' task separator
    iSepLength = Len(strSep)   ' used to figure out how many characters to skip between task records
    strClip = strClip + strSep ' UR 1.3 does not include these after last record
        
    ' iterate through clipboard text and create calendar items
    Dim iClipLen, iPosStart, iPosNext, iPosDuration As Integer
    Dim myURAppt As AppointmentItem
    'Dim varStart, varEnd As Variant
    Dim iMinutes, iDuration As Integer
    
    iClipLen = Len(strClip)
    iPosStart = 1
    
    ' extract header if there is one
    If InStr(1, strClip, "Item Title", vbTextCompare) = 1 Then
        ' extract header
        iPosNext = InStr(iPosStart, strClip, strSep, 1)
        Dim strHeader As String
        strHeader = Mid(strClip, 1, iPosNext) ' note: last CR is included in header extracted
        ' extract importance and duration column numbers
        Dim iPriColNum, iDurColNum, iTextColNum As Integer
        iPriColNum = ColumnNumber(strHeader, "Priority")
        iDurColNum = ColumnNumber(strHeader, "Duration")
        iTextColNum = ColumnNumber(strHeader, "Item Text")
        iPosStart = iPosNext + iSepLength ' skip over separator at end of line
    End If
    
    ' find first task
    iPosNext = InStr(iPosStart, strClip, strSep, vbTextCompare)
    
    ' extract tasks
    While iPosNext > 0
        ' extract task
        strTask = Mid(strClip, iPosStart, iPosNext + 1 - iPosStart) ' include CR at end of Task
                    
        ' extract task title
        strTitle = ColumnText(strTask, 1)
        
        ' extract importance
        If iDefaultImportance = 3 Then
            If iPriColNum > 0 Then
                iPriority = ColumnValue(strTask, iPriColNum)
                iImportance = Switch(iPriority = 1, olImportanceHigh, iPriority = 2, olImportanceNormal, True, olImportanceLow)
            Else
                iImportance = olImportanceLow
            End If
        Else
            iImportance = iDefaultImportance
        End If
        
        ' extract duration
        If iDurColNum > 0 Then
            iDuration = ColumnValue(strTask, iDurColNum)
        Else
            iDuration = iDefaultDuration
        End If
        If Not iDuration > 0 Then
            ' try to extract duration from title if found, else use default, then set End Time
            iPosDuration = InStr(1, strTitle, "~", vbTextCompare)
            If iPosDuration > 0 Then
                strDuration = Mid(strTitle, iPosDuration + 1)
                strTitle = Left(strTitle, iPosDuration - 1)
                iDuration = Val(strDuration)
            Else
                iDuration = iDefaultDuration
            End If
        End If
        varEnd = DateAdd("n", iDuration, varStart) 'add duration to indicate end time for task
        
        ' extract item text
        If iTextColNum > 0 Then
            strText = ColumnText(strTask, iTextColNum, True)
        Else
            strText = ""
        End If
        
        ' avoid any conflicts
        Dim strFilter As String
        Dim myAppts As Items
        Dim conflictAppts As Items
        Dim lastConflict As AppointmentItem
        Dim tempConflict As AppointmentItem
        Dim bConflict As Boolean
        Do
            bConflict = False
            ' CHECK TO MAKE SURE DAY AND TIME IS IN WORK TIME
            ' check day of week
            If Not abValidDays(Weekday(varStart)) Then
                varStart = DateAdd("d", 1, varStart)
                bConflict = True
            End If
            
            ' check hour
            If Not bConflict Then
                If TimeValue(varStart) < TimeValue(varBeginDay) Then
                    varStart = DateValue(varStart) + TimeValue(varBeginDay)
                    bConflict = True
                ElseIf TimeValue(varStart) > TimeValue(varEndDay) Then
                    varStart = DateAdd("d", 1, DateValue(varStart)) + TimeValue(varBeginDay)
                    bConflict = True
                ElseIf TimeValue(varEnd) > TimeValue(varEndDay) Or TimeValue(varEnd) < TimeValue(varBeginDay) Then
                    varStart = DateAdd("d", 1, DateValue(varStart)) + TimeValue(varBeginDay)
                    bConflict = True
                End If
            End If
            
            ' CHECK FOR CONFLICTS WITH EXISTING APPOINTMENTS
            If Not bConflict Then
                ' setup filter
                strStart = Format(varStart, "ddddd hh:mm AMPM")
                strEnd = Format(varEnd, "ddddd hh:mm AMPM")
                ' set filter criteria to include overlapping events that aren't market as Free
                strFilter = "([End] > '" & strStart & "' AND [Start] < '" & strEnd & "') AND ([BusyStatus] <> " & Str(olFree) & ")"
                ' old version: strFilter = "([End] > '" & strStart & "' AND [Start] < '" & strEnd & "') OR ([End] > '" & strStart & "' AND [End] <= '" & strEnd & "')"
                ' make sure recurring items are included
                Set myAppts = myNameSpace.GetDefaultFolder(olFolderCalendar).Items
                myAppts.Sort "[Start]", False
                myAppts.IncludeRecurrences = True
                ' find conflicting items and change start time to after last conflict
                Set conflictAppts = myAppts.Restrict(strFilter)
                ' Get last conflict (unfortunately can't use GetLast because IncludeRecurrences creates "infinite" collection
                ' so start from beginning and iterate until you get to the end
                Set lastConflict = Nothing
                Set tempConflict = conflictAppts.GetFirst
                Do Until tempConflict Is Nothing
                    Set lastConflict = tempConflict
                    Set tempConflict = conflictAppts.GetNext
                Loop
                If Not lastConflict Is Nothing Then
                    varStart = dtRoundUp(lastConflict.End, iDefaultDuration)
                    bConflict = True
                End If
            End If
            
            If bConflict Then varEnd = DateAdd("n", iDuration, varStart)
        Loop While bConflict
        
        'create task
        Set myURAppt = myOlApp.CreateItem(olAppointmentItem)
        With myURAppt
            .Subject = strTitle
            .Start = varStart
            .End = varEnd
            .Body = strText
            .Categories = "URTask"
            .Importance = iImportance
            .BusyStatus = iBusyStatus
            .Save
        End With
                
        varStart = varEnd  ' set start of next task to end of this
        
        ' find next task
        iPosStart = iPosNext + iSepLength ' skip over separator at end of task record
        If iPosStart < iClipLen Then
            iPosNext = InStr(iPosStart, strClip, strSep, vbTextCompare)
        Else
            iPosNext = 0
        End If
    Wend

End Sub

Sub ClearURAppts()
    Dim myOlApp As Application
    Dim myNameSpace As NameSpace
    Dim myCalendar As MAPIFolder
    Dim strFilter As String
    Dim myURTasks As Items
    
    Set myOlApp = CreateObject("Outlook.Application")
    Set myNameSpace = myOlApp.GetNamespace("MAPI")
    Set myCalendar = myNameSpace.GetDefaultFolder(olFolderCalendar)
    strFilter = "[Categories] = 'URTask'"
    Set myURTasks = myCalendar.Items.Restrict(strFilter)
    ' remove starting from the end (can't start from beginning because it messes up loop
    For iURTask = myURTasks.Count To 1 Step -1
        myURTasks.Remove iURTask
    Next

End Sub


Function dtRoundUp(dtTime As Date, iInterval As Integer) As Date
    ' Round the time value in varTime to the nearest minute
    ' interval in intInterval
    Dim iTime As Integer
    Dim sTime As Single
    Dim iHour As Integer
    Dim iMinute As Integer
    Dim lDate As Long
    ' Get the date portion of the date/time value
    lDate = DateValue(dtTime)
    ' Get the time portion as a number like 11.5 for 11:30.
    sTime = TimeValue(dtTime) * 24
    ' Get the hour and store it away. Int truncates,
    ' CInt rounds, so use Int.
    iHour = Int(sTime)
    ' Get the number of minutes, and then round up to the nearest
    ' occurrence of the interval specified.
    iMinute = CInt((sTime - iHour) * 60)
    iMinute = -Int(-iMinute / iInterval) * iInterval
    ' Build back up the original date/time value,
    ' rounded to the nearest interval.
    dtRoundUp = CDate(lDate + ((iHour + iMinute / 60) / 24))
End Function

Function ColumnNumber(strHeader As String, strColTitle As String) As Integer
    
    Dim vColPos
    vColPos = InStr(1, strHeader, strColTitle, vbTextCompare)
    Dim iCount As Integer
    iCount = 0
    If vColPos > 0 Then
        ' count how many tabs occur before column title
        Dim strBefore As String
        Dim vNextPos
        strBefore = Left(strHeader, vColPos - 1)
        vNextPos = 0
        Do
            iCount = iCount + 1
            If vNextPos = Len(strBefore) Then Exit Do
            vNextPos = InStr(vNextPos + 1, strBefore, Chr(9), vbTextCompare)
        Loop Until vNextPos = 0
    End If
    ColumnNumber = iCount
    
End Function

Function ColumnText(ByVal strTask As String, ByVal iColumn As Integer, Optional ByVal bExtractToEnd As Boolean = False) As String

    If iColumn > 0 Then
        ' find position of specified column number
        Dim vStartPos, vNextPos, vEndPos
        Dim iColCount As Integer
        vStartPos = 1
        iColCount = 1
        Do While iColCount < iColumn
            If vStartPos >= Len(strTask) Then Exit Do
            vNextPos = InStr(vStartPos, strTask, Chr(9), vbTextCompare) + 1
            If vNextPos > vStartPos Then vStartPos = vNextPos
            iColCount = iColCount + 1
        Loop
        
        ' find position of end of the column
        vEndPos = InStr(vStartPos, strTask, Chr(9), vbTextCompare)
        If vEndPos = 0 Or bExtractToEnd Then vEndPos = Len(strTask) ' no subsequent tab found or specified to extract to end, so use end of string
        
        ' extract the column text and convert it to a value
        ColumnText = Mid(strTask, vStartPos, vEndPos - vStartPos)
    Else
        ColumnText = ""
    End If
        
End Function

Function ColumnValue(ByVal strTask As String, ByVal iColumn As Integer) As Integer
    
    If iColumn > 0 Then
        ColumnValue = Val(ColumnText(strTask, iColumn))
    Else
        ColumnValue = 0
    End If

End Function
